/*
 * Matrices.java
 *
 * Creado el 9 de marzo de 2006, 18:18
 */


package auxiliar;

/**
 *
 * @author Vctor Jimnez, Antonio Pallars,
 * Mara Jos Cano Vicente, Juan Sebastin Daz Garca,
 * Bernardo Gonzlez Merino
 *
 * MTODOS NUMRICOS
 * LICENCIATURA DE MATEMTICAS
 * UNIVERSIDAD DE MURCIA
 */

public class Matrices {
        
    static double precision = 5E-16;
    
    public static boolean ifCuadrada(double[][] A) {
        int n = A.length;
        boolean respuesta = true;
        for(int i = 0; i < n; i++) {
            if(A[i].length != n) {
                respuesta = false;
                break;
            }
        }
        return respuesta;
    }
    
    public static boolean ifMatriz(double[][] A) {
        int n = A.length;
        int m = A[0].length;
        boolean respuesta = true;
        
        for(int i = 0; i < n; i++) {
            if(A[i].length != m) {
                respuesta = false;
                break;
            }
        }
        return respuesta;
    }
    
    public static double[][] suma(double[][] A, double[][] B)
            throws ErrorMetodosLineales {
        int n;
        int m;
        double[][] AmasB;
        
        if (!ifMatriz(A)) {
            System.out.println("Error: matriz izquierda mal definida");
            throw new ErrorMetodosLineales();
        }
        if (!ifMatriz(B)) {
            System.out.println("Error: matriz derecha mal definida");
            throw new ErrorMetodosLineales();
        }
        n = B.length;
        m = B[0].length;
        if ((n != A.length) || (m != A[0].length)) {
            System.out.println("Error: dimensiones incompatibles");
            throw new ErrorMetodosLineales();
        }
        
        AmasB = new double[n][m];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                AmasB[i][j] = A[i][j] + B[i][j];
            }
        }
        return AmasB;
    }
    
    public static double[] suma(double[] u, double[] v)
            throws ErrorMetodosLineales {
        double[][] A = new double [1][u.length];
        double[][] B = new double [1][v.length];
        
        A[0] = u;
        B[0] = v;
        return suma(A, B)[0];
    }
    
    public static double[][] resta(double[][] A, double[][] B)
            throws ErrorMetodosLineales {
        double[][] menosB = producto(-1, B);
        
        return suma(A, menosB);
    }
    
    public static double[] resta(double[] u, double[] v)
            throws ErrorMetodosLineales {
        double[] menosv = producto(-1, v);
        
        return suma(u, menosv);
    }
    
    public static double[][] producto(double[][] A, double[][] B)
            throws ErrorMetodosLineales {
        int n;
        int m;
        int p;
        double[][] AB;
        
        if (!ifMatriz(A)) {
            System.out.println("Error: matriz izquierda mal definida");
            throw new ErrorMetodosLineales();
        }
        if (!ifMatriz(B)) {
            System.out.println("Error: matriz derecha mal definida");
            throw new ErrorMetodosLineales();
        }
        n = A.length;
        m = B.length;
        p = B[0].length;
        if (m != A[0].length) {
            System.out.println("Error: dimensiones incompatibles");
            throw new ErrorMetodosLineales();
        }
        
        AB = new double[n][p];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < p; j++) {
                AB[i][j] = 0;
                for (int k = 0; k < m; k++) {
                    AB[i][j] = AB[i][j] + A[i][k] * B[k][j];
                }
            }
        }
        return AB;
    }
    
    public static double[][] producto(double c, double[][] A)
            throws ErrorMetodosLineales {
        int n;
        int m;
        double[][] cA;
        
        if (!ifMatriz(A)) {
            System.out.println("Error: matriz mal definida");
            throw new ErrorMetodosLineales();
        }
        
        n = A.length;
        m = A[0].length;
        cA = new double[n][m];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                cA[i][j] = c * A[i][j];
            }
        }
        return cA;
    }
    
    public static double[] producto(double c, double[] u)
            throws ErrorMetodosLineales {
        double[][] A = new double[1][u.length];
        
        A[0] = u;
        return producto(c, A)[0];
    }
    
    public static double producto(double[] u, double[] v)
            throws ErrorMetodosLineales {
        double[][] A = new double[1][u.length];
        double[][] B = new double[1][v.length];
        double[][] Bt = new double[v.length][1];
        
        A[0] = u;
        B[0] = v;
        Bt = traspuesta(B);
        
        return producto(A, Bt)[0][0];
    }
    
    public static double[] producto(double[][] A, double[] v)
            throws ErrorMetodosLineales {
        double[][] B = new double[1][v.length];
        double[][] Bt = new double[v.length][1];
        
        B[0] = v;
        Bt = traspuesta(B);
        return traspuesta(producto(A, Bt))[0];
    }
    
    public static double[][] traspuesta(double[][] A)
            throws ErrorMetodosLineales {
        int n;
        int m;
        double[][] At;
        
        if (!ifMatriz(A)) {
            System.out.println("Error: matriz mal definida");
            throw new ErrorMetodosLineales();
        }
        
        n = A.length;
        m = A[0].length;
        At = new double[m][n];
        for (int j = 0; j < m; j++) {
            for (int i = 0; i < n; i++) {
                At[j][i] = A[i][j];
            }
        }
        return At;
    }
    
    public static void escribe(double[][] A) {
        int n;
        String cadena;
        
        n = A.length;
        for (int i = 0; i < n; i++) {
            cadena = "(" + A[i][0];
            for (int j = 1; j < A[i].length; j++) {
                cadena = cadena + ", " + A[i][j];
            }
            cadena = cadena + ")";
            System.out.println(cadena);
        }
    }
    
    public static void escribe(double[] u) {
        double[][] A = new double[1][u.length];
        
        A[0] = u;
        escribe(A);
    }
    
    public static void escribe(int[][] A) {
        int n;
        String cadena;
        
        n = A.length;
        for (int i = 0; i < n; i++) {
            cadena = "(" + A[i][0];
            for (int j = 1; j < A[i].length; j++) {
                cadena = cadena + ", " + A[i][j];
            }
            cadena = cadena + ")";
            System.out.println(cadena);
        }
    }
    
    public static void escribe(int[] u) {
        int[][] A = new int[1][u.length];
        
        A[0] = u;
        escribe(A);
    }
    
    public static String toString(double[][] A) {
        int n;
        String cadena = "";
        
        n = A.length;
        for (int i = 0; i < n; i++) {
            cadena = cadena + "(" + A[i][0];
            for (int j = 1; j < A[i].length; j++) {
                cadena = cadena + ", " + A[i][j];
            }
            if (i < n - 1) {
                cadena = cadena + ")\n";
            } else {
                cadena = cadena + ")";
            }
        }
        return cadena;
    }
    
    public static String toString(double[] u) {
        double[][] A = new double[1][u.length];
        
        A[0] = u;
        return toString(A);
    }
    
    public static String toString(int[][] A) {
        int n;
        String cadena = "";
        
        n = A.length;
        for (int i = 0; i < n; i++) {
            cadena = cadena + "(" + A[i][0];
            for (int j = 1; j < A[i].length; j++) {
                cadena = cadena + ", " + A[i][j];
            }
            if (i < n - 1) {
                cadena = cadena + ")\n";
            } else {
                cadena = cadena + ")";
            }
        }
        return cadena;
    }
    
    public static String toString(int[] u) {
        int[][] A = new int[1][u.length];
        
        A[0] = u;
        return toString(A);
    }
    
    public static double norma(double[] u) {
        double norm = 0;
        
        for (int i = 0; i < u.length; i++) {
            norm = norm + u[i] * u[i];
        }
        return Math.sqrt(norm);
    }
    
    public static double norma1(double[][] A) throws ErrorMetodosLineales {
        int n = A.length;
        double norma;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        norma = 0;
        for(int j = 0; j < n; j++) {
            double suma = 0;
            for(int i = 0; i < n; i++) {
                suma = suma + Math.abs(A[i][j]);
            }
            if(norma < suma) {
                norma = suma;
            }
        }
        return norma;
    }

    public static double aproxRadioEspectral(double[][] A)
            throws ErrorMetodosLineales {
        double radio;
        double radioAprox = 0;
        int n = A.length;
        int k;
        double[][] IterA;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }       
        
        IterA = new double[n][n];
        for (int i = 0; i < n; i++) {
            for(int j = 0; j < n; j++) {
                IterA[i][j] = A[i][j];
            }
        }
        radio = Matrices.norma1(IterA);
        k = 1;
        while ( (radio != Double.NaN) && (radio != Double.POSITIVE_INFINITY)
                && (k <= 1000) )  {
            radioAprox = radio;
            System.out.println("||M^{" + k + "}||^{1/" + k + "} = " 
                    + radioAprox);
            k = k + 1;
            IterA = Matrices.producto(A, IterA);
            radio = Math.pow(Matrices.norma1(IterA), 1. / k);
        }
       
        return radioAprox;
    }
    
    public static double determinanteMenores(double[][] A)
            throws ErrorMetodosLineales {
        int n;
        int signo;
        double det;
        double[][] Aux;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        n = A.length;
        if (n == 1) {
            return A[0][0];
        } else {
            Aux = new double[n - 1][n - 1];
            det = 0.;
            signo = 1;
            for (int k = 0; k < n; k++) {
                for (int i = 0; i < k; i++) {
                    for (int j = 0; j < n - 1; j++) {
                        Aux[i][j] = A[i][j + 1];
                    }
                }
                for (int i = k; i < n - 1; i++) {
                    for (int j = 0; j < n - 1; j++) {
                        Aux[i][j] = A[i + 1][j + 1];
                    }
                }
                det = det + signo * A[k][0] * determinanteMenores(Aux);
                signo = - signo;
            }
            return det;
        }
        
    }
    
    public static double[] reglaCramer(double[][] A, double[] b)
            throws ErrorMetodosLineales {        
        int n = A.length;
        double[][] M;
        double[] x; 
        double denominador;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        
        denominador = determinanteMenores(A);
        if (Math.abs(denominador) < precision) {
            System.out.println("Error: matriz singular");
            throw new ErrorMetodosLineales();
        }
        
        M = new double[n][n];
        x = new double[n];
        
        for (int k = 0; k < n; k ++) {
            
            /* Calculamos el coeficiente x_k del vector solucin */
            for (int i = 0; i < n; i++) {
                for(int j = 0; j < n; j++) {
                    if (j != k) {
                        M[i][j] = A[i][j];
                    } else {
                        M[i][j] = b[i];
                    }                    
                }
            }
            x[k] = determinanteMenores(M) / denominador;
        }           
        return x;
    }
    
    public static double[][] sen(double[][] A, int iter) 
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] B = new double[n][n];
        double[][] senA = new double[n][n];
        
        if (!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
      
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                B[i][j] = A[i][j];
                senA[i][j] = A[i][j];
            }
        }
        for (int k = 1; k <= iter; k++) {
            B = producto(-1. / (2 * k * (2 * k + 1)), producto(A, producto(A, B)));            
            senA = suma(senA, B);      
        }        
        return senA;
    }
    
    public static double[][] cos(double[][] A, int iter) 
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] B = new double[n][n];
        double[][] cosA = new double[n][n];
        
        if (!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        for (int i = 0; i < n; i++) {
            B[i][i] = 1;
            cosA[i][i] = 1;
        }
        for (int k = 1; k <= iter; k++) {
            B = producto(-1. / ((2* k - 1) * 2 * k), producto(A, producto(A, B)));
            cosA = suma(cosA, B);
        }
        
        return cosA;
    }
    
    public static double[] gaussParcial(double[][] A, double[] b)
            throws ErrorMetodosLineales {
        int n = A.length;
        int m;
        int p;
        double mul;
        double[][] B; // matriz de los cambios
        double [] v; // vector de los cambios
        int[] s; // vector puntero
        double[] x; // vector donde escribir las soluciones
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        
        B = new double[n][n];
        v = new double[n];
        s = new int[n];
        x = new double[n];
        
        /* Condiciones iniciales */
        for (int i = 0; i < n; i++) {
            for(int j = 0; j < n; j++) {
                B[i][j] = A[i][j];
            }
        }
        for (int i = 0; i < n; i++) {
            v[i] = b[i];
        }
        for (int i = 0; i < n; i++) {
            s[i] = i;
        }
        
        /* Vamos a hacer las etapas k = 0, 1,..., n-1 */
        for (int k = 0; k < n; k++) {
            
            /* 1. Eleccin del pivote parcial */
            p = k; // iniciamos la bsqueda en la fila k
            for (int i = k + 1; i < n; i++) {
                if((Math.abs(B[s[i]][k])) > (Math.abs(B[s[p]][k]))) {
                    p = i; // apuntamos a la fila donde est el puntero
                }
            }
            if (Math.abs(B[s[p]][k]) < precision) {
                System.out.println("Error: matriz singular");
                System.out.println(toString(B));
                throw new ErrorMetodosLineales();
            }
            
            /* 2. Intercambio virtual de las filas pues cambiamos slo el puntero */
            m = s[k];
            s[k] = s[p];
            s[p] = m;
            
            /* 3. Eliminacin */
            for (int i = k + 1; i < n; i++) {
                mul = B[s[i]][k] / B[s[k]][k];
                B[s[i]][k] = 0; // asignamos ceros porque es el resultado de
                // las operaciones y as nos las ahorramos
                for(int j = k + 1; j < n; j++) {
                    B[s[i]][j] = B[s[i]][j] - mul * B[s[k]][j];
                }
                v[s[i]] = v[s[i]] - mul * v[s[k]];
            }
            
        } // ya tenemos la matriz de forma triangular superior
        
        /* Ahora resolvemos el sistema por el metodo ascendente */
        for (int k = n - 1; k >= 0; k--) {
            x[k] = v[s[k]];
            for (int j = k + 1; j < n; j++) {
                x[k] = x[k] - B[s[k]][j] * x[j];
            }
            x[k] = x[k] / B[s[k]][k];
        }
        return x;
    }
 
     public static double determinanteGauss(double[][] A)
            throws ErrorMetodosLineales {
        int n = A.length;
        int m;
        int p;
        int t = 0; // variable para guardar el nmero de cambios de fila
        double mul;
        double sol;
        double[][] B; // matriz de los cambios
        int[] s; // vector puntero
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        B = new double[n][n];
        s = new int[n];
        
        /* Condiciones iniciales */
        for (int i = 0; i < n; i++) {
            for(int j = 0; j < n; j++) {
                B[i][j] = A[i][j];
            }
        }
        for (int i = 0; i < n; i++) {
            s[i] = i;
        }
        
        /* Vamos a hacer las etapas k=0, 1,..., n-1 */
        for (int k = 0; k < n; k++) {
            
            // 1. Eleccin del pivote parcial
            p = k; // Iniciamos la bsqueda en la fila k
            for (int i = k + 1; i < n; i++) {
                if((Math.abs(B[s[i]][k])) > (Math.abs(B[s[p]][k]))) {
                    p = i; // apuntamos a la fila donde est el puntero
                }
            }
            if (Math.abs(B[s[p]][k]) < precision) {
                return 0;
            }
            /* 2. Intercambio virtual de las filas pues cambiamos slo el puntero */
            if(s[k] != s[p]) {
                t = t + 1; // contamos el cambio de las filas
            }
            m = s[k];
            s[k] = s[p];
            s[p] = m;
            
            /* 3. Eliminacin */
            for (int i = k + 1; i < n; i++) {
                mul = B[s[i]][k] / B[s[k]][k];
                B[s[i]][k] = 0; // asignamos ceros porque es el resultado
                // de las operaciones y as nos las ahorramos
                for(int j = k + 1; j < n; j++) {
                    B[s[i]][j] = B[s[i]][j] - mul * B[s[k]][j];
                }
            }
        } // ya tenemos la matriz de forma triangular superior
        
        /* Ahora multiplicamos los elementos de la diagonal */
        sol = 1;
        for (int k = 0; k < n; k++) {
            sol = sol * B[s[k]][k];
        }
        sol = Math.pow(-1, t) * sol;
        return sol;
    }
        
    public static double[][] inversa(double[][] A) throws ErrorMetodosLineales {
        int n = A.length;
        int m;
        int p;
        double mul;
        double[][] B; // matriz de los cambios
        double[][] C; // matriz inicialmente la identidad
        double[][] Inv; // matriz donde obtendremos la inversa
        int[] s; // vector puntero
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        B = new double[n][n];
        C = new double[n][n];
        Inv = new double[n][n];
        s = new int[n];
        
        /* Condiciones iniciales */
        for (int i = 0; i < n; i++) {
            for(int j = 0; j < n; j++) {
                B[i][j] = A[i][j];
            }
        }
        for (int i = 0; i < n; i++) {
            C[i][i] = 1;
            s[i] = i;
        }
        
        /* Vamos a hacer las etapas k = 0, 1,..., n-1 */
        for (int k = 0; k < n; k++) {
            
            /* 1. Eleccin del pivote parcial */
            p = k; // iniciamos la bsqueda en la fila k
            for (int i = k + 1; i < n; i++) {
                if((Math.abs(B[s[i]][k])) > (Math.abs(B[s[p]][k]))) {
                    p = i; // apuntamos a la fila donde est el puntero
                }
            }
            if (Math.abs(B[s[p]][k]) < precision) {
                System.out.println("Error: matriz singular");
                System.out.println(toString(B));
                throw new ErrorMetodosLineales();
            }
            
            /* 2. Intercambio virtual de las filas pues cambiamos slo el puntero */
            m = s[k];
            s[k] = s[p];
            s[p] = m;
            
            /* 3. Eliminacin */
            for (int i = 0; i < n; i++) {
                if (i != k ) {
                    mul = B[s[i]][k] / B[s[k]][k];
                    B[s[i]][k] = 0; // asignamos ceros porque es el resultado de
                                    // las operaciones y as nos las ahorramos
                    for(int j = k + 1; j < n; j++) {
                        B[s[i]][j] = B[s[i]][j] - mul * B[s[k]][j];
                    }
                    for(int j = 0; j < n; j++) {
                        C[s[i]][j] = C[s[i]][j] - mul * C[s[k]][j];
                    }
                }
            }
            
        } // ya tenemos la matriz diagonalizada
        
        /* Calculamos la matriz inversa */
        for (int i = 0; i < n; i++) {
            for(int j = 0; j < n; j++) {
                Inv [i][j] = C[s[i]][j] / B[s[i]][i];
            }
        }
        return Inv;
    }
    
    public static double[] gaussTotal(double[][] A, double[] b)
            throws ErrorMetodosLineales {
        int n = A.length;
        int m;
        int p;
        int q;
        double mul;
        double[][] B; // matriz de los cambios
        double[] v; // vector de los cambios
        int[] fila; // vector puntero (cambios en filas)
        int[] columna; // vector puntero (cambios en columnas)
        double[] x; // vector donde escribir las soluciones
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        
        B = new double[n][n];
        v = new double[n];
        fila = new int[n];
        columna = new int[n];
        x = new double[n];
        
        /* Condiciones iniciales */
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                B[i][j] = A[i][j];
            }
        }
        for (int i = 0; i < n; i++) {
            v[i] = b[i];
        }
        for (int i = 0; i < n; i++) {
            fila[i] = i;
            columna[i] = i;
        }
        
        /* Vamos a hacer las etapas k = 0, 1,..., n-1 */
        for (int k = 0; k < n; k++) {
            
            /* 1. Eleccin del pivote total */
            p = k; // iniciamos la bsqueda en la fila k
            q = k; // y la columna k
            for (int i = k; i < n; i++) {
                for (int j = k; j < n; j++) {
                    if(Math.abs(B[fila[i]][columna[j]])
                    > Math.abs(B[fila[p]][columna[q]])) {
                        p = i; // apuntamos a la fila donde est el puntero
                        q = j; // apuntamos a la columna donde est el puntero
                    }
                }
            }
            if (Math.abs(B[fila[p]][columna[q]]) < precision) {
                System.out.println("Error: matriz singular");
                System.out.println(toString(B));
                throw new ErrorMetodosLineales();
            }
            
            /* 2. Intercambio virtual de filas y columnas */
            m = fila[k];
            fila[k] = fila[p];
            fila[p] = m;
            m = columna[k];
            columna[k] = columna[q];
            columna[q] = m;
            
            /* 3. Eliminacin */
            for (int i = k + 1; i < n; i++) {
                mul = B[fila[i]][columna[k]] / B[fila[k]][columna[k]];
                B[fila[i]][columna[k]] = 0; // asignamos ceros porque sabemos
                                            // de antemano el resultado de
                                            // las operaciones
                for (int j = k + 1; j < n; j++) {
                    B[fila[i]][columna[j]] = B[fila[i]][columna[j]]
                            - mul * B[fila[k]][columna[j]];
                }
                v[fila[i]] = v[fila[i]] - mul * v[fila[k]];
            }
            
        } // ya se ha triangulado la matriz
        
        /* Ahora resolvemos el sistema */
        for (int k = n - 1; k >= 0; k--) {
            x[columna[k]] = v[fila[k]];
            for (int j = k + 1; j < n; j++) {
                x[columna[k]] = x[columna[k]]
                        - B[fila[k]][columna[j]] * x[columna[j]];
            }
            x[columna[k]] = x[columna[k]] / B[fila[k]][columna[k]];
        }
        return x;
    }

    /**
     * El mtodo de abajo para calcular el rango de una matriz n x m es 
     * recursivo (se llama a s mismo). 
     * 
     * Primero trasponemos si es necesario la matriz (esto no cambia el rango) 
     * para que el nmero de filas sea a lo sumo igual al de columnas. 
     * Luego procedemos como en el mtodo de
     * Gauss. Si no hay problemas y llegamos al final, el rango coincidir 
     * con el nmero n de filas. Si por el contrario hay una etapa 0 <= k < n 
     * en la que todos los coeficientes de la columna k por debajo de la diagonal 
     * principal se anulan (estamos suponiendo para simplificar que no han
     * habido permutaciones de filas), ello significa que los primeros k vectores
     * fila son linealmente independientes entre s y de los restantes, por 
     * lo que el rango de la matriz de partida es k ms el rango de la matriz 
     * formada por los n-k vectores de abajo. (Ntese que hay una excepcin
     * a esta regla, que es el caso en el que la matriz es cuadrada 
     * y todo ha ido bien justo hasta el ltimo momento, es decir, cuando 
     * aparece 0 en el coeficientes [n-1][n-1]; en ese caso no hay 
     * nueva matriz que construir y el rango es n-1.) Para calcular el rango de  
     * esta ltima matriz volvemos a llamar al mtodo (despus de suprimir las
     * k+1 primeras columnas, que son todo ceros) y as sucesivamente.
     */
    public static int rango(double[][] A) throws ErrorMetodosLineales {
        int n = A.length;
        int m = A[0].length;
        int p, pp; // variables int auxiliares
        double mul; // variable double auxiliar
        double[][] B; // matriz de los cambios
        int[] s; // vector puntero
        double[][] Aux; // matriz auxiliar para la recursin
        
        if(!ifMatriz(A)) {
            System.out.println("Error: matriz mal definida");
            throw new ErrorMetodosLineales();
        }
        
        /** 
         * Si es necesario trasponemos la matriz para que el nmero de filas
         * no supere al de columnas
         */
        if (n > m) {
            B = new double[m][n];
            for (int i = 0; i < n; i++) {
                for(int j = 0; j < m; j++) {
                    B[j][i] = A[i][j];
                }                                
            }
            p = n;
            n = m;
            m = p;
        } else {
            B = new double[n][m];
            for (int i = 0; i < n; i++) {
                for(int j = 0; j < m; j++) {
                    B[i][j] = A[i][j];
                }                                
            }            
        }
        
        s = new int[n];
        for (int i = 0; i < n; i++) {
            s[i] = i;
        }
        
        /* Vamos a hacer las etapas k = 0, 1,..., n-1 */
        for (int k = 0; k < n; k++) {
            
            /* 1. Eleccin del pivote parcial */
            p = k; // iniciamos la bsqueda en la fila k
            for (int i = k + 1; i < n; i++) {
                if((Math.abs(B[s[i]][k])) > (Math.abs(B[s[p]][k]))) {
                    p = i; // apuntamos a la fila donde est el puntero
                }
            }
            if (Math.abs(B[s[p]][k]) < precision) {
                if ((k == n - 1) && (n == m)) {
                    return n - 1;
                } else {   
                    Aux = new double[n - k][m - k - 1]; 
                    for (int i = 0; i < n - k; i++) {
                        for(int j = 0; j < m - k - 1; j++) {
                            Aux[i][j] = B[s[k + i]][k + j + 1];
                        }
                    } 
                    return k + rango(Aux);
                }
            }
            
            /* 2. Intercambio virtual de las filas pues cambiamos slo el puntero */
            pp = s[k];
            s[k] = s[p];
            s[p] = pp;
            
            /* 3. Eliminacin */
            for (int i = k + 1; i < n; i++) {
                mul = B[s[i]][k] / B[s[k]][k];
                B[s[i]][k] = 0; // asignamos ceros porque es el resultado de
                                // las operaciones y as nos las ahorramos
                for(int j = k + 1; j < m; j++) {
                    B[s[i]][j] = B[s[i]][j] - mul * B[s[k]][j];
                }
            }
            
        } // ya tenemos la matriz de forma triangular superior
        
        return n;
    }    
    
    public static boolean independencia(double[][] A) throws ErrorMetodosLineales {
        int n = A.length;
        
        if (rango(A) == n) {
            return true;
        } else {
            return false;
        }
    }  
    
    public static double[] solucionFactorQR(double[][] A, double[] b)
            throws ErrorMetodosLineales {                
        int n = A.length;
        double[][] B; // matriz de los cambios
        double[] w; // vector de los cambios
        double[] x; // vector donde escribir las soluciones       
        double[] v; // vector de Householder
        double aux = 0.;
        int signo = 1;
        double norma = 0.;
        double norma2V = 0.;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        
        B = new double[n][n];
        w = new double[n];
        x = new double[n];
        
        /* Condiciones iniciales */
        for (int i = 0; i < n; i++) {
            for(int j = 0; j < n; j++) {
                B[i][j] = A[i][j];
            }
        }
        for (int i = 0; i < n; i++) {
            w[i] = b[i];
        }
                
        /* Vamos a hacer las etapas k = 0, 1,..., n-1 */
        for (int k = 0; k < n; k++) {
            
            // hacemos sum (|B_ik|)
            aux = Math.abs(B[k][k]);
            for (int i = k + 1; i < n; i++) {
                aux = aux + Math.abs(B[i][k]);
            }
            if (aux < precision) {
                System.out.println("Matriz singular");
                throw new ErrorMetodosLineales();
            }
            if (aux > precision) {
                if (B[k][k] < 0) {
                    signo = -1;
                } else {
                    signo = 1;
                }
                
                // norma (B_kk,...,B_(n-1)k)
                norma = B[k][k] * B[k][k];
                for (int i = k + 1; i < n; i++) {
                    norma = norma + B[i][k] * B[i][k];
                }
                norma = Math.sqrt(norma);
                
                // vector de HouseHolder
                v = new double[n - k + 1];
                v[0] = B[k][k] + signo * norma;
                for (int i = 1; i < n - k; i++) {
                    v[i] = B[k + i][k];
                }
                norma2V = 2 * norma * norma 
                        + 2 * norma * signo * B[k][k];
                B[k][k] = -signo * norma;
                for (int i = k + 1; i < n; i++) {
                    B[i][k] = 0;
                }
                for (int j = k + 1; j < n; j++) {
                    aux = v[0] * B[k][j];
                    for (int i = 1; i < n - k; i++) {
                        aux = aux + v[i] * B[k + i][j];
                    }
                    aux = aux * 2 / norma2V;
                    for (int i = k; i < n; i++) {
                        B[i][j] = B[i][j] - aux * v[i - k];
                    }
                }
                aux = v[0] * w[k];
                for (int i = 1; i < n - k; i++) {
                    aux = aux + v[i] * w[k + i];
                }
                aux = aux * 2 / norma2V;
                for (int i = k; i < n; i++) {
                    w[i] = w[i] - aux * v[i - k];
                }
            }
        }
        
        // mtodo ascendente
        for (int k = n - 1; k >= 0; k--) {
            x[k] = w[k];
            for (int j = k + 1; j < n; j++) {
                x[k] = x[k] - B[k][j] * x[j];
            }
            x[k] = x[k] / B[k][k];
        }
        return x;        
    }
    
    /**
     * NOTA: El diseo del mtodo impide que puedan factorizarse
     * matrices singulares
     */
    public static double[][][] factLUDoolittle(double[][] A)
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] L; // matriz triangular inferior de la descomposicin LU
        double[][] U; // matriz triangular superior de la descomposicin LU
        double[][][] LU; // matriz de dimensiones [2][n][n] donde guardaremos
        // las dos matrices
        double aux;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        L = new double[n][n];
        U = new double[n][n];
        LU = new double[2][n][n];
        for (int k = 0; k < n; k++) {
            aux = A[k][k];
            for (int s = 0; s < k; s++) {
                aux = aux - L[k][s] * U[s][k];
            }
            if(Math.abs(aux) < precision) {
                System.out.println("Error: no hay factorizacion LU");
                throw new ErrorMetodosLineales();
            }
            L[k][k] = 1;
            U[k][k] = aux;
            for (int j = k + 1; j < n; j++) {
                aux = A[k][j];
                for (int s = 0; s < k; s++) {
                    aux = aux - L[k][s] * U[s][j];
                }
                U[k][j] = aux;
            }
            for (int i = k + 1; i < n; i++) {
                aux = A[i][k];
                for (int s = 0; s < k; s++) {
                    aux = aux - L[i][s] * U[s][k];
                }
                L[i][k] = aux / U[k][k];
            }
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                LU[0][i][j] = L[i][j];
                LU[1][i][j] = U[i][j];
            }
        }
        return LU;
    }
    
    /**
     * NOTA: El diseo del mtodo impide que puedan factorizarse
     * matrices singulares
     */
    public static double[][][] factLUCrout(double[][] A)
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] L; // matriz triangular inferior de la descomposicin LU
        double[][] U; // matriz triangular superior de la descomposicin LU
        double[][][] LU; // matriz de dimensiones [2][n][n] donde guardaremos
        // las dos matrices
        double aux;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        L = new double[n][n];
        U = new double[n][n];
        LU = new double[2][n][n];
        for (int k = 0; k < n; k++) {
            aux = A[k][k];
            for (int s = 0; s < k; s++) {
                aux = aux - L[k][s] * U[s][k];
            }
            if(Math.abs(aux) < precision) {
                System.out.println("Error: no hay factorizacion LU");
                throw new ErrorMetodosLineales();
            }
            L[k][k] = aux;
            U[k][k] = 1;
            for (int j = k + 1; j < n; j++) {
                aux = A[k][j];
                for (int s = 0; s < k; s++) {
                    aux = aux - L[k][s] * U[s][j];
                }
                U[k][j] = aux / L[k][k];
            }
            for (int i = k + 1; i < n; i++) {
                aux = A[i][k];
                for (int s = 0; s < k; s++) {
                    aux = aux - L[i][s] * U[s][k];
                }
                L[i][k] = aux;
            }
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                LU[0][i][j] = L[i][j];
                LU[1][i][j] = U[i][j];
            }
        }
        return LU;
    }
    
    /**
     * NOTA: El diseo del mtodo impide que puedan factorizarse
     * matrices singulares
     */
    public static double[][][] factLUCholesky(double[][] A)
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] S; // matriz triangular inferior de la descomposicin LU
        double[][][] LU; // matriz de dimensiones [2][n][n] donde guardaremos
        // las dos matrices
        double aux;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (Math.abs(A[i][j] - A[j][i]) > precision) {
                    System.out.println("Error: matriz no simtrica");
                    throw new ErrorMetodosLineales();
                }
            }
        }
        
        S = new double[n][n];
        LU = new double[2][n][n];
        for (int k = 0; k < n; k++) {
            aux = A[k][k];
            for (int s = 0; s < k; s++) {
                aux = aux - S[k][s] * S[k][s];
            }
            if(aux < precision) {
                System.out.println("Error: no hay factorizacion LU de Cholesky");
                throw new ErrorMetodosLineales();
            }
            S[k][k] = Math.sqrt(aux);
            for (int i = k + 1; i < n; i++) {
                aux = A[i][k];
                for (int s = 0; s < k; s++) {
                    aux = aux - S[i][s] * S[k][s];
                }
                S[i][k] = aux / S[k][k];
            }
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                LU[0][i][j] = S[i][j];
                LU[1][i][j] = S[j][i];
            }
        }
        return LU;
    }
    
    public static double[] resolverSistemaLUDoolittle(double[][] A,double[] b)
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] L;
        double[][] U;
        double[][][] LU;
        double[] x;
        double[] y;
        double[] z;
        double aux;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        
        L = new double[n][n];
        U = new double[n][n];
        LU = new double[2][n][n];
        x = new double[n];
        y = new double[n];
        z = new double[n];
        
        try {
            LU = factLUDoolittle(A);
            for (int i = 0; i < n; i++) {
                z[i] = b[i];
                for (int j = 0; j < n; j++) {
                    L[i][j] = LU[0][i][j];
                    U[i][j] = LU[1][i][j];
                }
            }
        } catch (ErrorMetodosLineales err) {
            System.out.println("Error: el sistema no se puede resolver"
                    + " por este mtodo");
            throw new ErrorMetodosLineales();
        }
        
        /* Resolvemos el sistema Ly = z por el mtodo descendente */
        for (int i = 0; i < n; i++) {
            y[i] = z[i];
            for (int j = 0; j < i; j++) {
                y[i] = y[i] - L[i][j] * y[j];
            }
        }
        
        /* Resolvemos el sistema Ux = y por el mtodo descendente */
        for (int i = n - 1; i >= 0; i--) {
            x[i] = y[i];
            for (int j = i + 1; j < n; j++) {
                x[i] = x[i] - U[i][j] * x[j];
            }
            x[i] = x[i] / U[i][i];
        }
        return x;
    }
    
    public static double[] resolverSistemaLUCrout(double[][] A,double[] b)
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] L;
        double[][] U;
        double[][][] LU;
        double[] x;
        double[] y;
        double[] z;
        double aux;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        
        L = new double[n][n];
        U = new double[n][n];
        LU = new double[2][n][n];
        x = new double[n];
        y = new double[n];
        z = new double[n];
        
        try {
            LU = factLUCrout(A);
            for (int i = 0; i < n; i++) {
                z[i] = b[i];
                for (int j = 0; j < n; j++) {
                    L[i][j] = LU[0][i][j];
                    U[i][j] = LU[1][i][j];
                }
            }
        } catch (ErrorMetodosLineales err) {
            System.out.println("Error: el sistema no se puede resolver"
                    + " por este mtodo");
            throw new ErrorMetodosLineales();
        }
        
        /* Resolvemos el sistema Ly = z por el mtodo descendente */
        for (int i = 0; i < n; i++) {
            y[i] = z[i];
            for (int j = 0; j < i; j++) {
                y[i] = y[i] - L[i][j] * y[j];
            }
            y[i] = y[i] / L[i][i];
        }
        
        /* Resolvemos el sistema Ux = y por el mtodo descendente */
        for (int i = n - 1; i >= 0; i--) {
            x[i] = y[i];
            for (int j = i + 1; j < n; j++) {
                x[i] = x[i] - U[i][j] * x[j];
            }
        }
        return x;
    }
    
    public static double[] resolverSistemaLUCholesky(double[][] A,double[] b)
            throws ErrorMetodosLineales {
        int n = A.length;
        double[][] L;
        double[][] U;
        double[][][] LU;
        double[] x;
        double[] y;
        double[] z;
        double aux;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        
        L = new double[n][n];
        U = new double[n][n];
        LU = new double[2][n][n];
        x = new double[n];
        y = new double[n];
        z = new double[n];
        
        try {
            LU = factLUCholesky(A);
            for (int i = 0; i < n; i++) {
                z[i] = b[i];
                for (int j = 0; j < n; j++) {
                    L[i][j] = LU[0][i][j];
                    U[i][j] = LU[1][i][j];
                }
            }
        } catch (ErrorMetodosLineales err) {
            System.out.println("Error: el sistema no se puede resolver"
                    + " por este mtodo");
            throw new ErrorMetodosLineales();
        }
        
        /* Resolvemos el sistema Ly = z por el mtodo descendente */
        for (int i = 0; i < n; i++) {
            y[i] = z[i];
            for (int j = 0; j < i; j++) {
                y[i] = y[i] - L[i][j] * y[j];
            }
            y[i] = y[i] / L[i][i];
        }
        
        /* Resolvemos el sistema Ux = y por el mtodo descendente */
        for (int i = n - 1; i >= 0; i--) {
            x[i] = y[i];
            for (int j = i + 1; j < n; j++) {
                x[i] = x[i] - U[i][j] * x[j];
            }
            x[i] = x[i] / U[i][i];
        }
        return x;
    }
    
    public static double[][] pasarTridiagonalACuadrada(double[][] F)
            throws ErrorMetodosLineales {
        int n;
        double[][] A;
        
        if (F.length != 3) {
            System.out.println("Error: los datos no corresponden "
                    + "a una matriz tridiagonal");
            throw new ErrorMetodosLineales();
        }
        n = F[1].length;
        if ( (F[0].length != n - 1) || (F[2].length != n - 1) ) {
            System.out.println("Error: los datos no corresponden "
                    + "a una matriz tridiagonal");
            throw new ErrorMetodosLineales();
        }
        
        A = new double[n][n];
        for (int k = 0; k < n - 1; k++) {
            A[k + 1][k] = F[0][k];
            A[k][k + 1] = F[2][k];
        }
        for (int k = 0; k < n; k++) {
            A[k][k] = F[1][k];
        }
        return A;
    }
    
    /**
     * NOTA: El diseo del mtodo impide que puedan factorizarse
     * matrices singulares
     */
    public static double[][] factTridiagonal(double[][] F)
            throws ErrorMetodosLineales {
        int n;
        double[] a; // diagonal inferior de la matriz tridiagonal
        double[] b; // diagonal principal de la matriz tridiagonal
        double[] c; // diagonal superior de la matriz tridiagonal
        double[] r; // cocientes de los menores principales
        double[][] sol; // sol[0] es la diagonal inferior de L
        // sol[1] es la diagonal principal de U
        // sol[2] es la diagonal superior de U
        
        if (F.length != 3) {
            System.out.println("Error: los datos no corresponden "
                    + "a una matriz tridiagonal");
            throw new ErrorMetodosLineales();
        }
        n = F[1].length;
        if ( (F[0].length != n - 1) || (F[2].length != n - 1) ) {
            System.out.println("Error: los datos no corresponden "
                    + "a una matriz tridiagonal");
            throw new ErrorMetodosLineales();
        }
        
        a = new double[n - 1];
        b = new double[n];
        c = new double[n - 1];
        r = new double[n];
        sol = new double[3][n];
        sol[0] = new double[n - 1];
        sol[2] = new double[n - 1];
        
        for (int k = 0; k < n - 1; k++) {
            a[k] = F[0][k];
            c[k] = F[2][k];
        }
        for (int k = 0; k < n; k++) {
            b[k] = F[1][k];
        }
        
        r[0] = b[0];
        if(Math.abs(r[0]) < precision) {
            System.out.println("Error: no hay factorizacion LU");
            System.out.println("Menor principal nulo de dimensin 1");
            throw new ErrorMetodosLineales();
        }
        for(int k = 1; k < n; k++) {
            r[k] = b[k] - a[k - 1] * c[k - 1] / r[k - 1];
            if(Math.abs(r[k]) < precision) {
                System.out.println("Error: no hay factorizacion LU");
                System.out.println("Menor principal nulo de dimensin "
                        + (k + 1));
                throw new ErrorMetodosLineales();
            }
        }
        
        for(int k = 0; k < n; k++) {
            sol[1][k] = r[k];
        }
        for(int k = 0; k < n - 1; k++) {
            sol[0][k] = a[k] / r[k];
            sol[2][k] = c[k];
        }
        
        return sol;
    }
    
    public static double[] resolverSistemaTridiagonal
            (double[][] F, double[] b) throws ErrorMetodosLineales {
        int n; // dimensin de la matriz
        double[] l; // diagonal inferior de la matriz L
        double[] p; // diagonal principal de la matriz U
        double[] u; // diagonal superior de la matriz U
        double[] x; // solucin del sistema Ly = z
        double[] y; // solucin del sistema Ux = y
        double[] z; // trmino independiente
        double[][] G;
        if (F.length != 3) {
            System.out.println("Error: los datos no corresponden "
                    + "a una matriz tridiagonal");
            throw new ErrorMetodosLineales();
        }
        n = F[1].length;
        if ( (F[0].length != n - 1) || (F[2].length != n - 1) ) {
            System.out.println("Error: los datos no corresponden "
                    + "a una matriz tridiagonal");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones incompatibles");
            throw new ErrorMetodosLineales();
        }
        
        l = new double[n - 1];
        p = new double[n];
        u = new double[n - 1];
        x = new double[n];
        y = new double[n];
        z = new double[n];
        
        try {
            G = factTridiagonal(F);
            for (int k = 0; k < n; k++) {
                z[k] = b[k];
                p[k] = G[1][k];
            }
            for (int k = 0; k < n - 1; k++) {
                l[k] = G[0][k];
                u[k] = G[2][k];
            }
        } catch (ErrorMetodosLineales err) {
            System.out.println("Error: el sistema no se puede resolver"
                    + " por este mtodo");
            throw new ErrorMetodosLineales();
        }
        
        /* Resolvemos el sistema Ly = z por el mtodo descendente */
        y[0] = z[0];
        for (int k = 1; k < n; k++) {
            y[k] = z[k] - l[k - 1] * y[k - 1];
        }
        
        /* Resolvemos el sistema Ux = y por el mtodo ascendente */
        x[n - 1] = y[n - 1] / p[n - 1];
        for (int k = n - 2; k >= 0; k--) {
            x[k] = (y[k] - u[k] * x[k + 1]) / p[k];
        }
        return x;
    }
    
    public static double[] iterJacobi(double[][] A, double[] b, double tol,
            int nmax, double[] x0, boolean imp) throws ErrorMetodosLineales {
        int n = A.length;
        double xa[] = new double[n]; // vector para aproximar la solucin
        double e[] = new double[n]; // vector de correccin
        double norma; // norma de la correcin
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        if (x0.length != n) {
            System.out.println("Error: vector inicial mal dimensionado");
            throw new ErrorMetodosLineales();
        }
        for(int i = 0; i < n; i++) {
            xa[i] = x0[i];
            if(Math.abs(A[i][i]) < precision) {
                System.out.println("Error: Jacobi no es aplicable");
                throw new ErrorMetodosLineales();
            }
        }
        
        if (imp) {
            System.out.println("Vector inicial: " + toString(xa));
        }
        for(int k = 1; k <= nmax; k++) {
            norma = 0;
            for(int i = 0; i < n; i++) {
                e[i] = b[i];
                for(int j = 0; j < n; j++) {
                    e[i] = e[i] - A[i][j] * xa[j];
                }
                e[i] = e[i] / A[i][i];
                norma = norma + Math.abs(e[i]);
            }
            for(int i = 0; i < n; i++) {
                xa[i] = xa[i] + e[i];
            }
            if (imp) {
                System.out.println("Iteracin " + k + ": " + toString(xa));
            }
            if (norma < tol) {
                System.out.println("Jacobi converge en " + k + " iteradas");
                return xa;
            }
        }
        
        System.out.println("Error: no hay convergencia en " + nmax
                + " iteraciones");
        throw new ErrorMetodosLineales();
    }
    
    public static double[] iterJacobi(double[][] A, double[] b, double tol,
            int nmax) throws ErrorMetodosLineales {
        double[] x0 = new double[A.length];
        boolean imp = false;
        
        for(int i = 0; i < A.length; i++) {
            x0[i] = 1;
        }
        return iterJacobi(A, b, tol, nmax, x0, false);
    }
    
    public static double[] iterGaussSeidel(double[][] A, double[] b, double tol,
            int nmax, double[] x0, boolean imp) throws ErrorMetodosLineales {
        int n = A.length;
        double xa[] = new double[n]; // vector para aproximar la solucin
        double e[] = new double[n]; // vector de correccin
        double norma; // norma de la correcin
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        if (x0.length != n) {
            System.out.println("Error: vector inicial mal dimensionado");
            throw new ErrorMetodosLineales();
        }
        for(int i = 0; i < n; i++) {
            xa[i] = x0[i];
            if(Math.abs(A[i][i]) < precision) {
                System.out.println("Error: Gauss-Seidel no es aplicable");
                throw new ErrorMetodosLineales();
            }
        }
        
        if (imp) {
            System.out.println("Vector inicial: " + toString(xa));
        }
        for(int k = 1; k <= nmax; k++) {
            norma = 0;
            for(int i = 0; i < n; i++) {
                e[i] = b[i];
                for(int j = 0; j < n; j++) {
                    e[i] = e[i] - A[i][j] * xa[j];
                }
                e[i] = e[i] / A[i][i];
                norma = norma + Math.abs(e[i]);
                xa[i] = xa[i] + e[i];
            }
            if (imp) {
                System.out.println("Iteracin " + k + ": " + toString(xa));
            }
            if (norma < tol) {
                System.out.println("Gauss-Seidel converge en " + k
                        + " iteradas");
                return xa;
            }
        }
        
        System.out.println("Error: no hay convergencia en " + nmax
                + " iteraciones");
        throw new ErrorMetodosLineales();
    }
    
    public static int[] iterJacobiOperaciones(double[][] A, double[] b,
            double tol,  int nmax, double[] x0, boolean imp)
            throws ErrorMetodosLineales {
        int n = A.length;
        double xa[] = new double[n]; // vector para aproximar la solucin
        double e[] = new double[n]; // vector de correccin
        double norma; // norma de la correcin
        int[] operaciones = new int[3];
        int sumasYrestas = 0;
        int multiplicaciones = 0;
        int divisiones = 0;
        
        if(!ifCuadrada(A)) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (b.length != n) {
            System.out.println("Error: dimensiones no compatibles");
            throw new ErrorMetodosLineales();
        }
        if (x0.length != n) {
            System.out.println("Error: vector inicial mal dimensionado");
            throw new ErrorMetodosLineales();
        }
        for(int i = 0; i < n; i++) {
            xa[i] = x0[i];
            if(Math.abs(A[i][i]) < precision) {
                System.out.println("Error: Jacobi no es aplicable");
                throw new ErrorMetodosLineales();
            }
        }
        
        if (imp) {
            System.out.println("Vector inicial: " + toString(xa));
        }
        for(int k = 1; k <= nmax; k++) {
            norma = 0;
            for(int i = 0; i < n; i++) {
                e[i] = b[i];
                for(int j = 0; j < n; j++) {
                    e[i] = e[i] - A[i][j] * xa[j];
                    sumasYrestas = sumasYrestas + 1;
                    multiplicaciones = multiplicaciones + 1;
                }
                e[i] = e[i] / A[i][i];
                divisiones = divisiones + 1;
                norma = norma + Math.abs(e[i]);
                sumasYrestas = sumasYrestas + 1;
            }
            for(int i = 0; i < n; i++) {
                xa[i] = xa[i] + e[i];
                sumasYrestas = sumasYrestas + 1;
            }
            if (imp) {
                System.out.println("Iteracin " + k + ": " + toString(xa));
            }
            if (norma < tol) {
                operaciones[0] = sumasYrestas;
                operaciones[1] = multiplicaciones;
                operaciones[2] = divisiones;
                return operaciones;
            }
        }
        System.out.println("Error: no hay convergencia en " + nmax
                + " iteraciones");
        throw new ErrorMetodosLineales();
    }
    
    public static double[] iterGaussSeidel(double[][] A, double[] b, double tol,
            int nmax) throws ErrorMetodosLineales {
        double[] x0 = new double[A.length];
        boolean imp = false;
        
        for(int i = 0; i < A.length; i++) {
            x0[i] = 1;
        }
        return iterGaussSeidel(A, b, tol, nmax, x0, false);
    }
    
    public static double[][] iterInversa(double[][] A, double[][] X0,
            double tol, int nmax) throws ErrorMetodosLineales {
        int n = A.length;
        double[][] Id;
        double[][] InvAprox;
        double norma;
        
        if (X0.length != n) {
            System.out.println("Error: dimensiones incompatible");
            throw new ErrorMetodosLineales();
        }
        for (int i = 0; i < n; i++) {
            if(!ifCuadrada(A)) {
                System.out.println("Error: matriz no cuadrada");
                throw new ErrorMetodosLineales();
            }
            if(!ifCuadrada(X0)) {
                System.out.println("Error: matriz condicin inicial X0 "
                        + "no cuadrada");
                throw new ErrorMetodosLineales();
            }
        }
        InvAprox = new double[n][n];
        Id = new double[n][n];
        for (int i = 0; i < n; i++) {
            Id[i][i] = 1;
        }
        for(int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                InvAprox[i][j] = X0[i][j];
            }
        }
        
        for(int k = 1; k <= nmax; k++) {
            InvAprox = Matrices.suma(InvAprox,
                    Matrices.producto(InvAprox,
                    Matrices.resta(Id, Matrices.producto(A, InvAprox))));
            
            norma = 0;
            for(int i = 0; i < n; i++) {
                for (int j = 0; j < n; j++) {
                    norma = norma + Math.abs(
                            Matrices.resta(
                            Matrices.producto(InvAprox, A), Id)[i][j]);
                }
            }
            if (norma < tol) {
                System.out.println("El mtodo converge en " + k
                        + " iteradas");
                return InvAprox;
            }
        }
        
        System.out.println("Error: no hay convergencia en " + nmax
                + " iteraciones");
        throw new ErrorMetodosLineales();
    }  
    
    public static double[] potencia(double[][] A, double tol, int nmax,
            double[] v, double[] y) throws ErrorMetodosLineales {
        int n = A.length;        
        double[] sol;
        double[] uini;
        double[] ufin;
        double rini;
        double rfin;
                
        if (ifCuadrada(A) == false) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }
        if (v.length != n) {
            System.out.println("Error: vector inicial mal dimensionado");
            throw new ErrorMetodosLineales();
        }
        if (norma(v) < precision) {
            System.out.println("Error: vector inicial nulo");
            throw new ErrorMetodosLineales();
        }
        if (y.length != n) {
            System.out.println("Error: vector para el producto escalar" 
                    + " mal dimensionado");
            throw new ErrorMetodosLineales();
        }
        
        sol = new double[n + 1];
        uini = new double[n];
        ufin = new double[n];
        rini = 0;
        uini = producto (1 / norma(v), v);
        
        for (int m = 1; m <= nmax; m++) {
            if (Math.abs(producto(uini, y)) < tol) {
                System.out.println("Error: el mtodo de la potencia no es"
                        + " aplicable");
                throw new ErrorMetodosLineales();
            }
            ufin = producto(A, uini);
            if (norma(ufin) < precision) {
                System.out.println("Error: el mtodo de la potencia no es"
                        + " aplicable");
                throw new ErrorMetodosLineales();
            }
            rfin = producto(ufin, y) / producto(uini, y);
            ufin = producto (1 / norma(ufin), ufin);
            if (Math.abs(rfin - rini) < tol) {
                for (int i = 0; i < n; i++) {
                    sol[i] = ufin[i];
                }
                sol[n] = rfin;
                System.out.println("El mtodo de la potencia converge en "
                        + m + " iteradas");
                return sol;
            }
            rini = rfin;
            for (int i = 0; i < n; i++) {
                uini[i] = ufin[i];
            }
        }
        System.out.println("Error: no hay convergencia en " + nmax +
                " iteraciones");
        throw new ErrorMetodosLineales();
    }

    public static double[] potenciaInversa(double[][] A, double tol, int nmax,
            double[] v, double[] y) throws ErrorMetodosLineales {
        int n = A.length;        
        double[] sol; 
        double[] uini;
        double[] ufin;
        double rini;
        double rfin;
        double[][] L; 
        double[][] U;
        double[][][] LU; 
        double[] inter;
                
        if (ifCuadrada(A) == false) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        }        
        LU = factLUDoolittle(A);        
        if (v.length != n) {
            System.out.println("Error: vector inicial mal dimensionado");
            throw new ErrorMetodosLineales();
        }
        if (norma(v) < precision) {
            System.out.println("Error: vector inicial nulo");
            throw new ErrorMetodosLineales();
        }
        if (y.length != n) {
            System.out.println("Error: vector para el producto escalar" 
                    + " mal dimensionado");
            throw new ErrorMetodosLineales();
        }
        
        sol = new double[n + 1];
        uini = new double[n];
        ufin = new double[n];
        rini = 0;
        uini = producto (1 / norma(v), v);
        L = new double[n][n]; 
        U = new double[n][n];
        inter = new double[n];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                L[i][j] = LU[0][i][j];
                U[i][j] = LU[1][i][j];
            }
        }
        
        for (int m = 1; m <= nmax; m++) {
            if (Math.abs(producto(uini, y)) < tol) {
                System.out.println("Error: el mtodo de la potencia no es"
                        + " aplicable");
                throw new ErrorMetodosLineales();
            }
            
            /**
             * Resolvemos A ufin = uini resolviendo primero L inter = uini 
             * por el mtodo descendente y luego U ufin  = inter por el 
             * mtodo ascendente.
             */
            for (int i = 0; i < n; i++) {
                inter[i] = uini[i];
                for (int j = 0; j < i; j++) {
                    inter[i] = inter[i] - L[i][j] * inter[j];
                }
            }
            for (int i = n - 1; i >= 0; i--) {
                ufin[i] = inter[i];
                for (int j = i + 1; j < n; j++) {
                    //OJO: aqu estaba el error!
                    ufin[i] = ufin[i] - U[i][j] * ufin[j];
                }
                ufin[i] = ufin[i] / U[i][i];
            }
            
            if (norma(ufin) < precision) {
                System.out.println("Error: el mtodo de la potencia inversa"
                        + " no es aplicable");
                throw new ErrorMetodosLineales();
            }
            rfin = producto(ufin, y) / producto(uini, y);
            ufin = producto (1 / norma(ufin), ufin);
            if (Math.abs(rfin - rini) < tol) {
                for (int i = 0; i < n; i++) {
                    sol[i] = ufin[i];
                }
                //OJO: aqu haba otro error
                sol[n] = 1 / rfin;
                System.out.println("El mtodo de la potencia inversa " 
                        + "converge en " + m + " iteradas");
                return sol;
            }
            rini = rfin;
            for (int i = 0; i < n; i++) {
                uini[i] = ufin[i];
            }
        }
        System.out.println("Error: no hay convergencia en " + nmax +
                " iteraciones");
        throw new ErrorMetodosLineales();
    }    
   
    /**
     * El metodo potenciaMasProxima devuelve un vector sol cuya ltima 
     * coordenada es el valor propio de una matriz A ms prximo a un
     * numero dado y las primeras coordenadas las del correspondiente
     * vector propio.
     */
    public static double[] potenciaMasProxima(double[][] A, double tol, int nmax,
            double[] v, double[] y, double lambda0) throws ErrorMetodosLineales {
        int n = A.length;
        double[][] InvdeAmenoslambda0 = new double[n][n];
        double[][] Id = new double[n][n];
        double[] sol = new double[n];
        
        for (int i = 0; i < n; i++) {
            Id[i][i] = 1;
        }
        
        try {
            InvdeAmenoslambda0 = Matrices.inversa(
                    Matrices.resta(
                    A, Matrices.producto(lambda0, Id)
                    )
                    );
            sol = potencia(InvdeAmenoslambda0, tol, nmax, v, y);
            
            /**
             * Si mu es valor propio de mayor valor absoluto para 
             * (A - lambda0 Id)^{-1}, entonces el valor propio lambda que
             * necesitamos viene dado por mu = 1 / (lambda - lambda0),
             * es decir, lambda = 1 / mu + lambda0. Por eso es preciso
             * hacer el cambio de abajo. Ntese que el vector propio no
             * cambia.
             */
            sol[4] = 1 / sol[4] + lambda0;
            return sol;
        } catch (ErrorMetodosLineales err) {
            System.out.println("Error: el mtodo no funciona");
            throw new ErrorMetodosLineales();
        }
    }   
    
    public static double[][] deflacionWielandt(double[][] A, double tol, 
            int nmax) throws ErrorMetodosLineales {
        int n = A.length;
        double[][] Sol;
        double[][] B;
        double[][] Soldefl;
        double[] vectvaldom;
        double[] v;
        double[] y;
        double[] u;
        double[] x;
        double[] w;
        int k;
        double lambda;
        double lambdadefl;
        double aux;
        
        if (ifCuadrada(A) == false) {
            System.out.println("Error: matriz no cuadrada");
            throw new ErrorMetodosLineales();
        } 
        
        Sol = new double[n][n + 1];
        if (n == 1) {
            Sol[0][0] = 1;
            Sol[0][1] = A[0][0];           
        } else {
            B = new double[n - 1][n - 1];
            Soldefl = new double[n - 1][n];
            vectvaldom = new double[n + 1];
            v = new double[n];
            y = new double[n];
            u = new double[n];
            x = new double[n];
            w = new double[n];
            
            for (int i = 0; i < n; i++) {
                v[i] = Math.random();
                y[i] = Math.random();
            }            
            
            vectvaldom = potencia(A, tol, nmax, v, y);
            for (int i = 0; i < n; i++) {
                u[i] = vectvaldom[i];
            }
            lambda = vectvaldom[n];
            
            aux = 0;
            k = 0;
            for (int i = 0; i < n; i++) {
                if (Math.abs(u[i]) > Math.abs(aux)) {
                    aux = u[i];
                    k = i;
                }                  
            }
            for (int j = 0; j < n; j++) {
                x[j] = A[k][j] / (lambda * aux);
            }                      
            for (int i = 0; i < n - 1; i++) {
                for (int j = 0; j < n - 1; j++) {
                    if ((i < k) && (j < k)) {
                        B[i][j] = A[i][j] - lambda * u[i] * x[j];
                    } else if (i < k) {
                        B[i][j] = A[i][j + 1] - lambda * u[i] * x[j + 1];
                    } else if (j < k) {
                        B[i][j] = A[i + 1][j] - lambda * u[i + 1] * x[j];
                    } else {
                        B[i][j] = A[i + 1][j + 1] - lambda * u[i + 1] * x[j + 1];
                    }
                }
            }
            Soldefl = deflacionWielandt(B, tol, nmax);
            if (Math.abs(Soldefl[n - 2][n - 1] - lambda) < tol) {                 
                System.out.println("Error: valor propio mltiple");
                throw new ErrorMetodosLineales();
            }
            
            for (int i = 0; i < n - 1; i++) {
                for (int j = 0; j < k; j++) {
                    w[j] = Soldefl[i][j];                    
                } 
                w[k] = 0;
                for (int j = k + 1; j < n; j++) {
                    w[j] = Soldefl[i][j - 1];                    
                }
                lambdadefl = Soldefl[i][n - 1];                
                aux = producto(x, w);
                for (int j = 0; j < n; j++) {
                    w[j] = (lambdadefl - lambda) * w[j] + lambda * aux * u[j];                             
                }
                aux = norma(w);
                for (int j = 0; j < n; j++) {
                    Sol[i][j] =  w[j] / aux;
                }
                Sol[i][n] = lambdadefl;
            }
            for (int j = 0; j < n; j++) {
                Sol[n - 1][j] = u[j];
            }
            Sol[n - 1][n] = lambda;
        }
        
        return Sol;
    }

    public static double[][] jacobiPropios(double[][] A, double tol, int nmax)
            throws ErrorMetodosLineales {
         int n = A.length;                  
         double[][] B;
         double[][] O; // matriz auxiliar donde se guardan los vectores propios
                       // aproximados 
         double[][] Sol; // matriz para devolver los resultados
         double aux;
         int p = 0;
         int q = 0;
         double x;
         double t;
         double c;
         double s;
         boolean alerta;

         for (int i = 0; i < n; i++) {
             if (ifCuadrada(A) == false) {
                 System.out.println("Error: matriz no cuadrada");
                 throw new ErrorMetodosLineales();
             }
         }
         B = new double[n][n];
         O = new double[n][n];
         Sol = new double[n][n + 1];
         for (int i = 0; i < n; i++) {
             O[i][i] = 1; 
             for (int j = 0; j < n; j++) {
                 B[i][j] = A[i][j];                 
             }             
         }
                  
         for (int k = 1; k <= nmax; k++) {             
             aux = 0;
             for (int i = 0; i < n; i++) {
                 for (int j = i + 1; j < n; j++) {
                     if (Math.abs(B[i][j]) > aux) {
                         p = i;
                         q = j;
                         aux = Math.abs(B[i][j]);
                     }
                 }
             }
             if (aux < tol) {
                 alerta = false;
                 for (int i = 0; i < n; i++) {
                     for (int j = i + 1; j < n; j++) {
                         if (Math.abs(B[i][i] - B[j][j]) < tol) {
                             alerta = true;                                 
                         }
                     }
                 }
                 if (alerta) {
                    System.out.println("Advertencia: posible error"
                            + " en los vectores propios");  
                 }
                 for (int i = 0; i < n; i++) {
                     for (int j = 0; j < n; j++) {
                         Sol[i][j] = O[j][i]; 
                     }
                     Sol[i][n] = B[i][i]; 
                 }
                 System.out.println("Jacobi (valores y vectores propios) " 
                         + "converge en " + k + " iteradas");
                 return Sol;
             }
             
             x = (B[q][q] - B[p][p]) / (2 * B[p][q]); 
             if (x >= 0) {
                 t = - 1 * x + Math.sqrt(1 + x * x);
             } else {
                 t = - 1 * x - Math.sqrt(1 + x * x);
             }
             aux = Math.sqrt(1 + t * t);
             c = 1 / aux;
             s = t / aux;
             B[p][p] = B[p][p] - t * B[p][q];
             B[q][q] = B[q][q] + t * B[p][q];
             B[p][q] = 0;
             B[q][p] = 0;
             for (int i = 0; i < n; i++) {
                 if ( (i != p) && (i != q)) {                     
                     aux = B[i][p];
                     B[i][p] = c * aux - s * B[i][q];
                     B[i][q] = s * aux + c * B[i][q];   
                     B[p][i] = B[i][p];
                     B[q][i] = B[i][q];
                 }
                 aux = O[i][p];
                 O[i][p] = c * aux - s * O[i][q];
                 O[i][q] = s * aux + c * O[i][q];
             }             
         }
         System.out.println("Error: no hay convergencia en " + nmax +
                 " iteraciones");
         throw new ErrorMetodosLineales();
     }
    
    /**
     * Excepcin ErrorMetodosLineales para ser devuelta por
     * los mtodos de la clase
     */
    public static class  ErrorMetodosLineales extends Exception {
        ErrorMetodosLineales() {
            super("Error: mtodos lineales");
        }
    }
}
